@@ -1,37 +1,23 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
-#include <jdm.h>
+#include <xfs/xfs.h>
+#include <xfs/jdm.h>
 
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -49,7 +35,6 @@
 #include "media.h"
 #include "arch_xlate.h"
 
-#ifdef RMT
 /* this rmt junk is here because the rmt protocol supports writing ordinary
  * (non-device) files in the remote /dev directory! yuck!
  */
@@ -66,7 +51,6 @@ extern int rmtioctl( int, int, ... );
 extern int rmtopen( char *, int, ... );
 extern int rmtread( int, void*, uint);
 extern int rmtwrite( int, const void *, uint);
-#endif
 
 
 /* drive_simple.c - drive strategy for standard in or a file
@@ -289,7 +273,7 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 				        S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
 		if ( contextp->dc_fd < 0 ) {
 			mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-			      "unable to open %s: %s\n",
+			      _("unable to open %s: %s\n"),
 			      drivep->d_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
@@ -303,7 +287,7 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 		if ( rval ) {
 			if ( errno != ENOENT ) {
 				mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-				      "stat of %s failed: %s\n",
+				      _("stat of %s failed: %s\n"),
 				      drivep->d_pathname,
 				      strerror( errno ));
 				return BOOL_FALSE;
@@ -333,8 +317,8 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 				break;
 			default:
 				mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-				      "cannot dump to %s "
-				      "file type %x\n",
+				      _("cannot dump to %s "
+				      "file type %x\n"),
 				      drivep->d_pathname,
 				      statbuf.st_mode & S_IFMT );
 				return BOOL_FALSE;
@@ -344,7 +328,7 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 #ifdef RESTORE
 		if ( rval ) {
 			mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-			      "stat of %s failed: %s\n",
+			      _("stat of %s failed: %s\n"),
 			      drivep->d_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
@@ -363,8 +347,7 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 			break;
 		default:
 			mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-			      "cannot restore from %s "
-			      "file type %x\n",
+			      _("cannot restore from %s file type %x\n"),
 			      drivep->d_pathname,
 			      statbuf.st_mode & S_IFMT );
 			return BOOL_FALSE;
@@ -375,7 +358,7 @@ ds_instantiate( int argc, char *argv[], drive_t *drivep, bool_t singlethreaded )
 				        S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
 		if ( contextp->dc_fd < 0 ) {
 			mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-			      "unable to open %s: %s\n",
+			      _("unable to open %s: %s\n"),
 			      drivep->d_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
@@ -523,7 +506,7 @@ do_begin_read( drive_t *drivep )
 	 */
 	if ( ! global_hdr_checksum_check( tmphdr )) {
 		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-		      "media file header checksum error\n" );
+		      _("media file header checksum error\n") );
 		free(tmphdr);
 		return DRIVE_ERROR_CORRUPTION;
 	}
@@ -541,7 +524,7 @@ do_begin_read( drive_t *drivep )
 	 */
 	if ( strncmp( grhdrp->gh_magic, GLOBAL_HDR_MAGIC, GLOBAL_HDR_MAGIC_SZ)) {
 		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-		      "media file header magic number mismatch: %s, %s\n",
+		      _("media file header magic number mismatch: %s, %s\n"),
 		      grhdrp->gh_magic,
 		      GLOBAL_HDR_MAGIC);
 		return DRIVE_ERROR_FORMAT;
@@ -551,7 +534,7 @@ do_begin_read( drive_t *drivep )
 	 */
 	if ( global_version_check( grhdrp->gh_version ) != BOOL_TRUE ) {
 		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-		      "unrecognized media file header version (%d)\n",
+		      _("unrecognized media file header version (%d)\n"),
 		      grhdrp->gh_version );
 		return DRIVE_ERROR_VERSION;
 	}
@@ -560,8 +543,8 @@ do_begin_read( drive_t *drivep )
 	 */
 	if ( drhdrp->dh_strategyid != drive_strategy_simple.ds_id ) {
 		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-		      "unrecognized drive strategy ID "
-		      "(media says %d, expected %d)\n",
+		      _("unrecognized drive strategy ID "
+		      "(media says %d, expected %d)\n"),
 		      drhdrp->dh_strategyid, drive_strategy_simple.ds_id );
 		return DRIVE_ERROR_FORMAT;
 	}
@@ -921,8 +904,7 @@ do_begin_write( drive_t *drivep )
 		rval = ftruncate( contextp->dc_fd, 0 );
 		if ( rval ) {
 			mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-			      "attempt to truncate %s failed: "
-			      "%d (%s)\n",
+			      _("attempt to truncate %s failed: %d (%s)\n"),
 			      drivep->d_pathname,
 			      errno,
 			      strerror( errno ));
@@ -972,7 +954,7 @@ do_begin_write( drive_t *drivep )
 
 	if ( ! global_hdr_checksum_check( tmphdr )) {
 		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_DRIVE,
-		      "media file header checksum error\n" );
+		      _("media file header checksum error\n") );
 	}
 	else {
 		mlog( MLOG_NITTY, "media file header checksum OK!\n" );
@@ -1084,7 +1066,7 @@ do_set_mark( drive_t *drivep,
 			newoff = lseek64( contextp->dc_fd, ( off64_t )0, SEEK_SET );
 			if ( newoff < 0 ) {
 				mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-				      "could not save first mark: %d (%s)\n",
+				      _("could not save first mark: %d (%s)\n"),
 				      errno,
 				      strerror( errno ));
 			} else {
@@ -1288,7 +1270,7 @@ do_write( drive_t *drivep, char *bufp, size_t writesz )
 				  sizeof( contextp->dc_buf ));
 		if ( nwritten < 0 ) {
 			mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-			      "write to %s failed: %d (%s)\n",
+			      _("write to %s failed: %d (%s)\n"),
 			      drivep->d_pathname,
 			      errno,
 			      strerror( errno ));
@@ -1382,7 +1364,7 @@ do_end_write( drive_t *drivep, off64_t *ncommittedp )
 				  remaining_bufsz );
 		if ( nwritten < 0 ) {
 			mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-			      "write to %s failed: %d (%s)\n",
+			      _("write to %s failed: %d (%s)\n"),
 			      drivep->d_pathname,
 			      errno,
 			      strerror( errno ));
@@ -1436,7 +1418,7 @@ do_rewind( drive_t *drivep )
 	if ( newoff ) {
 		ASSERT( newoff < 0 );
 		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-		      "could not rewind %s: %s\n",
+		      _("could not rewind %s: %s\n"),
 		      drivep->d_pathname,
 		      strerror( errno ));
 		return DRIVE_ERROR_DEVICE;
@@ -1472,7 +1454,7 @@ do_erase( drive_t *drivep )
 	if ( newoff ) {
 		ASSERT( newoff < 0 );
 		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-		      "could not rewind %s in prep for erase: %s\n",
+		      _("could not rewind %s in prep for erase: %s\n"),
 		      drivep->d_pathname,
 		      strerror( errno ));
 		return DRIVE_ERROR_DEVICE;
@@ -1483,7 +1465,7 @@ do_erase( drive_t *drivep )
 	rval = ftruncate64( contextp->dc_fd, ( off64_t )0 );
 	if ( rval ) {
 		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_DRIVE,
-		      "could not erase %s: %s (%d)\n",
+		      _("could not erase %s: %s (%d)\n"),
 		      drivep->d_pathname,
 		      strerror( errno ),
 		      errno );
