@@ -118,6 +118,28 @@ static int mlog_main_exit_code = -1;
 static rv_t mlog_main_exit_return = RV_NONE;
 static rv_t mlog_main_exit_hint = RV_NONE;
 
+void
+mlog_init0( void )
+{
+	int i;
+
+#ifdef DUMP
+	mlog_fp = stderr;
+#endif /* DUMP */
+#ifdef RESTORE
+	mlog_fp = stdout;
+#endif /* RESTORE */
+
+	/* initialize stream count. will be updated later by call to
+	 * mlog_tell_streamcnt( ), after drive layer has counted drives
+	 */
+	mlog_streamcnt = 1;
+
+	for( i = 0 ; i < MLOG_SS_CNT ; i++ ) {
+		mlog_level_ss[ i ] = MLOG_VERBOSE;
+	}
+}
+
 bool_t
 mlog_init1( intgen_t argc, char *argv[ ] )
 {
@@ -127,18 +149,6 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 	size_t vsymcnt;
 	intgen_t c;
 
-#ifdef DUMP
-        mlog_fp = stderr;
-#endif /* DUMP */
-#ifdef RESTORE
-        mlog_fp = stdout;
-#endif /* RESTORE */
-
-	/* initialize stream count. will be updated later by call to
-	 * mlog_tell_streamcnt( ), after drive layer has counted drives
-	 */
-	mlog_streamcnt = 1;
-
 	/* prepare an array of suboption token strings. this will be the
 	 * concatenation of the subsystem names with the verbosity symbols.
 	 * this array of char pts must be null terminated for getsubopt( 3 ).
@@ -179,7 +189,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 				fprintf( stderr,
 					 _("%s: -%c argument missing\n"),
 					 progname,
-					 optopt );
+					 c );
 				usage( );
 				return BOOL_FALSE;
 			}
@@ -195,7 +205,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 					fprintf( stderr,
 						 _("%s: -%c argument invalid\n"),
 						 progname,
-						 optopt );
+						 c );
 					usage( );
 					return BOOL_FALSE;
 				}
@@ -210,7 +220,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 							 "%s requires a "
 							 "verbosity value\n"),
 							 progname,
-							 optopt,
+							 c,
 						mlog_ss_names[ suboptix ] );
 						usage( );
 						return BOOL_FALSE;
@@ -225,7 +235,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 							 "does not require "
 							 "a value\n"),
 							 progname,
-							 optopt );
+							 c );
 						usage( );
 						return BOOL_FALSE;
 					}
@@ -238,7 +248,7 @@ mlog_init1( intgen_t argc, char *argv[ ] )
 						 _("%s: -%c argument "
 						 "invalid\n"),
 						 progname,
-						 optopt );
+						 c );
 					usage( );
 					return BOOL_FALSE;
 				}
@@ -447,11 +457,6 @@ mlog_va( intgen_t levelarg, char *fmt, va_list args )
 	}
 }
 
-
-static const char *exit_strings[] =
-	{ "SUCCESS", "ERROR", "INTERRUPT", "", "FAULT" };
-
-
 /*
  * Map RV codes to actual error messages.
  */
@@ -574,7 +579,7 @@ _mlog_exit( const char *file, int line, int exit_code, rv_t rv )
 	      "%s: %d: mlog_exit called: "
 	      "exit_code: %s return: %s (%s)\n",
 	      file, line,
-	      exit_strings[exit_code],
+	      exit_codestring(exit_code),
 	      rvp->rv_string, rvp->rv_desc);
 
 	if (rv < 0 || rv >= _RV_NUM) {
@@ -770,12 +775,7 @@ mlog_exit_flush(void)
 	if (interrupt) status_str = "INTERRUPT";
 	else if (quit) status_str = "QUIT";
 	else if (incomplete) status_str = "INCOMPLETE";
-#ifdef NDEBUG
-	/* We should never get here, but if we do make sure we don't die
-	   horribly when not running debug. */
-	else if (! VALID_EXIT_CODE(mlog_main_exit_code)) status_str = "UNKNOWN";
-#endif /* NDEBUG */
-	else status_str = exit_strings[mlog_main_exit_code];
+	else status_str = exit_codestring(mlog_main_exit_code);
 
 	/* now print the overall state of the dump/restore */
 	fprintf(mlog_fp, "%s: %s Status: %s\n", progname, PROGSTR_CAPS, status_str);
