@@ -1,37 +1,33 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
-#include <jdm.h>
+/*
+ * Directory attributes are written on tape during the directory dump phase,
+ * which precedes the non-dir dump phase. The directory attributes cannot be
+ * restored, however, until all of the non-dir files have been restored
+ * (directory timestamps would be wrong, directory inherit flags would
+ * interfere, etc.) These routines allow the directory attributes to be stored
+ * on disk when they are read from the dump stream so that they can be applied
+ * at a later time.
+ */
+
+#include <xfs/xfs.h>
+#include <xfs/jdm.h>
 
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -115,18 +111,15 @@ struct dirattr {
 	time32_t d_ctime;
 	u_int32_t d_xflags;
 	u_int32_t d_extsize;
+	u_int32_t d_projid;
 	u_int32_t d_dmevmask;
 	u_int32_t d_dmstate;
-#ifdef EXTATTR
 	off64_t d_extattroff;
-#endif /* EXTATTR */
 };
 
 typedef struct dirattr dirattr_t;
 
-#ifdef EXTATTR
 #define DIRATTR_EXTATTROFFNULL	( ( off64_t )OFF64MAX )
-#endif /* EXTATTR */
 
 /* dirattr persistent context definition
  */
@@ -140,17 +133,20 @@ typedef struct dirattr_pers dirattr_pers_t;
 
 /* dirattr transient context definition
  */
+
+#define	DIRATTR_BUFSIZE	32768
+
 struct dirattr_tran {
 	char *dt_pathname;
 	int dt_fd;
 	bool_t dt_at_endpr;
 	dah_t dt_cachedh;
 	dirattr_t dt_cached_dirattr;
-#ifdef EXTATTR
+	size_t dt_off;
+	char dt_buf[DIRATTR_BUFSIZE];
 	char *dt_extattrpathname;
 	int dt_extattrfd;
 	bool_t dt_extattrfdbadpr;
-#endif /* EXTATTR */
 };
 
 typedef struct dirattr_tran dirattr_tran_t;
@@ -179,9 +175,7 @@ extern size_t pgsz;
 /* forward declarations of locally defined static functions ******************/
 
 static void dirattr_get( dah_t );
-#ifdef EXTATTR
-static void dirattr_flush( void );
-#endif /* EXTATTR */
+static void dirattr_cacheflush( void );
 #ifdef DIRATTRCHK
 static u_int16_t calcdixcum( dix_t dix );
 #endif /* DIRATTRCHK */
@@ -193,9 +187,7 @@ static u_int16_t calcdixcum( dix_t dix );
 /* definition of locally defined static variables *****************************/
 
 static char *dirattrfile = "dirattr";
-#ifdef EXTATTR
 static char *dirextattrfile = "dirextattr";
-#endif /* EXTATTR */
 static dirattr_tran_t *dtp = 0;
 static dirattr_pers_t *dpp = 0;
 
@@ -205,11 +197,9 @@ static dirattr_pers_t *dpp = 0;
 bool_t
 dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 {
-#ifdef SESSCPLT
 	if ( dtp ) {
 		return BOOL_TRUE;
 	}
-#endif /* SESSCPLT */
 
 	/* sanity checks
 	 */
@@ -223,9 +213,7 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	ASSERT( dtp );
 	dtp->dt_cachedh = DAH_NULL;
 	dtp->dt_fd = -1;
-#ifdef EXTATTR
 	dtp->dt_extattrfd = -1;
-#endif /* EXTATTR */
 
 	/* generate a string containing the pathname of the dirattr file
 	 */
@@ -238,9 +226,9 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 		 */
 		dtp->dt_fd = open( dtp->dt_pathname, O_RDWR );
 		if ( dtp->dt_fd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
 			      "could not find directory attributes file %s: "
-			      "%s\n",
+			      "%s\n"),
 			      dtp->dt_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
@@ -254,28 +242,24 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 				   O_RDWR | O_CREAT | O_EXCL,
 				   S_IRUSR | S_IWUSR );
 		if ( dtp->dt_fd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
 			      "could not create directory attributes file %s: "
-			      "%s\n",
+			      "%s\n"),
 			      dtp->dt_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
 		}
 
-		/* reserve space for the backing store. try to use F_RESVSP64.
-		 * if doesn't work, try F_RESVSP64. the former is faster, since
+		/* reserve space for the backing store. try to use RESVSP64.
+		 * if doesn't work, try ALLOCSP64. the former is faster, as
 		 * it does not zero the space.
 		 */
 		{
 		bool_t successpr;
-		intgen_t ioctlcmd;
+		unsigned int ioctlcmd;
 		intgen_t loglevel;
 		size_t trycnt;
 
-#ifndef F_RESVSP64
-#define F_RESVSP64 0
-#endif /* F_RESVSP64 */
-
 		for ( trycnt = 0,
 		      successpr = BOOL_FALSE,
 		      ioctlcmd = XFS_IOC_RESVSP64,
@@ -303,17 +287,17 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 			rval = ioctl( dtp->dt_fd, ioctlcmd, &flock64 );
 			if ( rval ) {
 				if ( errno != ENOTTY ) {
-					mlog( loglevel | MLOG_NOTE,
+					mlog( loglevel | MLOG_NOTE, _(
 					      "attempt to reserve %lld bytes for %s "
 					      "using %s "
-					      "failed: %s (%d)\n",
+					      "failed: %s (%d)\n"),
 					      initsz,
 					      dtp->dt_pathname,
-					      ioctlcmd == F_RESVSP64
+					      ioctlcmd == XFS_IOC_RESVSP64
 					      ?
-					      "F_RESVSP64"
+					      "XFS_IOC_RESVSP64"
 					      :
-					      "F_ALLOCSP64",
+					      "XFS_IOC_ALLOCSP64",
 					      strerror( errno ),
 					      errno );
 				}
@@ -332,8 +316,8 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 				        ( off_t )0 );
 	ASSERT( dpp );
 	if ( dpp == ( dirattr_pers_t * )-1 ) {
-		mlog( MLOG_NORMAL | MLOG_ERROR,
-		      "unable to map %s: %s\n",
+		mlog( MLOG_NORMAL | MLOG_ERROR, _(
+		      "unable to map %s: %s\n"),
 		      dtp->dt_pathname,
 		      strerror( errno ));
 		return BOOL_FALSE;
@@ -349,7 +333,6 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	 */
 	dtp->dt_at_endpr = BOOL_FALSE;
 
-#ifdef EXTATTR
 	/* calculate the dir extattr pathname, and set the fd to -1.
 	 * file will be created on demand.
 	 */
@@ -359,7 +342,6 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 	if ( resume ) {
 		( void )unlink( dtp->dt_extattrpathname );
 	}
-#endif /* EXTATTR */
 
 	return BOOL_TRUE;
 }
@@ -386,7 +368,6 @@ dirattr_cleanup( void )
 		( void )unlink( dtp->dt_pathname );
 		free( ( void * )dtp->dt_pathname );
 	}
-#ifdef EXTATTR
 	if ( dtp->dt_extattrfd >= 0 ) {
 		( void )close( dtp->dt_extattrfd );
 		dtp->dt_extattrfd = -1;
@@ -395,7 +376,7 @@ dirattr_cleanup( void )
 		( void )unlink( dtp->dt_extattrpathname );
 		free( ( void * )dtp->dt_extattrpathname );
 	}
-#endif /* EXTATTR */
+
 	free( ( void * )dtp );
 	dtp = 0;
 }
@@ -404,7 +385,6 @@ dah_t
 dirattr_add( filehdr_t *fhdrp )
 {
 	dirattr_t dirattr;
-	intgen_t nwritten;
 	off64_t oldoff;
 	dix_t dix;
 #ifdef DIRATTRCHK
@@ -423,8 +403,8 @@ dirattr_add( filehdr_t *fhdrp )
 		off64_t newoff;
 		newoff = lseek64( dtp->dt_fd, dpp->dp_appendoff, SEEK_SET );
 		if ( newoff == ( off64_t )-1 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
-			      "lseek of dirattr failed: %s\n",
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
+			      "lseek of dirattr failed: %s\n"),
 			      strerror( errno ));
 			return DAH_NULL;
 		}
@@ -432,6 +412,12 @@ dirattr_add( filehdr_t *fhdrp )
 		dtp->dt_at_endpr = BOOL_TRUE;
 	}
 
+	if (dtp->dt_off + sizeof(dirattr_t) > sizeof(dtp->dt_buf)) {
+		if (dirattr_flush() != RV_OK) {
+			return DAH_NULL;
+		}
+	}
+
 	/* calculate the index of this dirattr
 	 */
 	oldoff = dpp->dp_appendoff;
@@ -448,6 +434,7 @@ dirattr_add( filehdr_t *fhdrp )
 	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
+	dirattr.d_projid = fhdrp->fh_stat.bs_projid;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
 	dirattr.d_dmstate = ( u_int32_t )fhdrp->fh_stat.bs_dmstate;
 #ifdef DIRATTRCHK
@@ -455,24 +442,17 @@ dirattr_add( filehdr_t *fhdrp )
 	sum = calcdixcum( dix );
 	dirattr.d_sum = sum;
 #endif /* DIRATTRCHK */
-#ifdef EXTATTR
 	dirattr.d_extattroff = DIRATTR_EXTATTROFFNULL;
-#endif /* EXTATTR */
 
-	/* write the dirattr
+	/* write the entry into our buffer
 	 */
-	nwritten = write( dtp->dt_fd, ( void * )&dirattr, sizeof( dirattr ));
-	if ( ( size_t )nwritten != sizeof( dirattr )) {
-		mlog( MLOG_NORMAL | MLOG_ERROR,
-		      "write of dirattr failed: %s\n",
-		      strerror( errno ));
-		return DAH_NULL;
-	}
+	memcpy(dtp->dt_buf + dtp->dt_off, (void *)&dirattr, sizeof(dirattr_t));
+	dtp->dt_off += sizeof(dirattr_t);
 
 	/* update the next write offset
 	 */
-	ASSERT( dpp->dp_appendoff <= OFF64MAX - ( off64_t )nwritten );
-	dpp->dp_appendoff += ( off64_t )nwritten;
+	ASSERT( dpp->dp_appendoff <= OFF64MAX - ( off64_t )sizeof(dirattr_t) );
+	dpp->dp_appendoff += ( off64_t )sizeof(dirattr_t);
 
 #ifdef DIRATTRCHK
 	dah = HDLMKHDL( sum, dix );
@@ -483,7 +463,6 @@ dirattr_add( filehdr_t *fhdrp )
 	return dah;
 }
 
-#ifdef EXTATTR
 void
 dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 {
@@ -508,10 +487,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 					  O_RDWR | O_CREAT,
 					  S_IRUSR | S_IWUSR );
 		if ( dtp->dt_extattrfd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not open/create directory "
 			      "extended attributes file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -527,10 +506,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	while ( off != DIRATTR_EXTATTROFFNULL ) {
 		seekoff = lseek64( dtp->dt_extattrfd, off, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -545,10 +524,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 			      ( void * )&off,
 			      sizeof( off ));
 		if ( nread != ( intgen_t )sizeof( off )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -561,10 +540,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	 */
 	off = lseek64( dtp->dt_extattrfd, 0, SEEK_END );
 	if ( off < 0 ) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not seek to end of extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -576,10 +555,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 			  ( void * )&nulloff,
 			  sizeof( nulloff ));
 	if ( nwritten != ( intgen_t )sizeof( nulloff )) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not write extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -588,10 +567,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	}
 	nwritten = write( dtp->dt_extattrfd, ( void * )ahdrp, ahdrp->ah_sz );
 	if ( nwritten != ( intgen_t )( ahdrp->ah_sz )) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not write at end of extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -604,14 +583,14 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	 */
 	if ( oldoff == DIRATTR_EXTATTROFFNULL ) {
 		dtp->dt_cached_dirattr.d_extattroff = off;
-		dirattr_flush( );
+		dirattr_cacheflush( );
 	} else {
 		seekoff = lseek64( dtp->dt_extattrfd, oldoff, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -623,10 +602,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 				  ( void * )&off,
 				  sizeof( off ));
 		if ( nwritten != ( intgen_t )sizeof( off )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not write extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -659,10 +638,10 @@ dirattr_cb_extattr( dah_t dah,
 					  O_RDWR | O_CREAT,
 					  S_IRUSR | S_IWUSR );
 		if ( dtp->dt_extattrfd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not open/create directory "
 			      "extended attributes file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -685,10 +664,10 @@ dirattr_cb_extattr( dah_t dah,
 		 */
 		seekoff = lseek64( dtp->dt_extattrfd, off, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -703,10 +682,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )&nextoff,
 			      sizeof( nextoff ));
 		if ( nread != ( intgen_t )sizeof( nextoff )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -720,10 +699,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )ahdrp,
 			      EXTATTRHDR_SZ );
 		if ( nread != EXTATTRHDR_SZ ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -739,10 +718,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )&ahdrp[ 1 ],
 			      recsz - EXTATTRHDR_SZ );
 		if ( nread != ( intgen_t )( recsz - EXTATTRHDR_SZ )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -764,7 +743,6 @@ dirattr_cb_extattr( dah_t dah,
 
 	return BOOL_TRUE;
 }
-#endif /* EXTATTR */
 
 void
 dirattr_update( dah_t dah, filehdr_t *fhdrp )
@@ -806,12 +784,19 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	ASSERT( dtp->dt_cached_dirattr.d_sum == sum );
 #endif /* DIRATTRCHK */
 
+	if ( dtp->dt_at_endpr && dtp->dt_off ) {
+		if (dirattr_flush() != RV_OK) {
+			assert( 0 );
+			return;
+		}
+	}
+
 	/* seek to the dirattr
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -827,18 +812,17 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
+	dirattr.d_projid = fhdrp->fh_stat.bs_projid;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
 	dirattr.d_dmstate = ( u_int32_t )fhdrp->fh_stat.bs_dmstate;
-#ifdef EXTATTR
 	dirattr.d_extattroff = DIRATTR_EXTATTROFFNULL;
-#endif /* EXTATTR */
 
 	/* write the dirattr
 	 */
 	nwritten = write( dtp->dt_fd, ( void * )&dirattr, sizeof( dirattr ));
 	if ( ( size_t )nwritten != sizeof( dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "update of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "update of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -909,6 +893,13 @@ dirattr_get_extsize( dah_t dah )
 	return dtp->dt_cached_dirattr.d_extsize;
 }
 
+u_int32_t
+dirattr_get_projid( dah_t dah )
+{
+	dirattr_get( dah );
+	return dtp->dt_cached_dirattr.d_projid;
+}
+
 u_int32_t
 dirattr_get_dmevmask( dah_t dah )
 {
@@ -923,6 +914,37 @@ dirattr_get_dmstate( dah_t dah )
 	return dtp->dt_cached_dirattr.d_dmstate;
 }
 
+rv_t
+dirattr_flush()
+{
+	ssize_t nwritten;
+
+	/* sanity checks
+	*/
+	assert ( dtp );
+
+	if (dtp->dt_off) {
+		/* write the accumulated dirattr entries
+		*/
+		nwritten = write( dtp->dt_fd, ( void * )dtp->dt_buf, dtp->dt_off);
+		if ( nwritten != dtp->dt_off ) {
+			if ( nwritten < 0 ) {
+				mlog( MLOG_NORMAL | MLOG_ERROR,
+					_("write of dirattr buffer failed: %s\n"),
+					strerror( errno ));
+			} else {
+				mlog( MLOG_NORMAL | MLOG_ERROR,
+					_("write of dirattr buffer failed: "
+					"expected to write %ld, actually "
+					"wrote %ld\n"), dtp->dt_off, nwritten);
+			}
+			assert( 0 );
+			return RV_UNKNOWN;
+		}
+		dtp->dt_off = 0;
+	}
+	return RV_OK;
+}
 
 /* definition of locally defined static functions ****************************/
 
@@ -965,12 +987,19 @@ dirattr_get( dah_t dah )
 	ASSERT( argoff >= ( off64_t )DIRATTR_PERS_SZ );
 	ASSERT( argoff <= dpp->dp_appendoff - ( off64_t )sizeof( dirattr_t ));
 
+	if ( dtp->dt_at_endpr && dtp->dt_off ) {
+		if (dirattr_flush() != RV_OK) {
+			assert( 0 );
+			return;
+		}
+	}
+
 	/* seek to the dirattr
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -982,8 +1011,8 @@ dirattr_get( dah_t dah )
 		      ( void * )&dtp->dt_cached_dirattr,
 		      sizeof( dtp->dt_cached_dirattr ));
 	if ( ( size_t )nread != sizeof( dtp->dt_cached_dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "read of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "read of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -997,9 +1026,8 @@ dirattr_get( dah_t dah )
 	dtp->dt_cachedh = dah;
 }
 
-#ifdef EXTATTR
 static void
-dirattr_flush( void )
+dirattr_cacheflush( void )
 {
 	dah_t dah;
 	dix_t dix;
@@ -1047,8 +1075,8 @@ dirattr_flush( void )
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -1060,16 +1088,14 @@ dirattr_flush( void )
 			  ( void * )&dtp->dt_cached_dirattr,
 			  sizeof( dtp->dt_cached_dirattr ));
 	if ( ( size_t )nwritten != sizeof( dtp->dt_cached_dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "flush of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "flush of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
 
 	dtp->dt_at_endpr = BOOL_FALSE;
 }
-#endif /* EXTATTR */
-
 
 #ifdef DIRATTRCHK
 
